//AssigningTool.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

import swarm.collections.ListImpl;
import swarm.collections.ListIndex;

/**
 * A tool to choose the unit to which we have to send an order
 *
 * @author Pietro Terna
 */
public class AssigningTool extends SwarmObjectImpl
{
    /** the list of the normal units existing in our environment*/
    public ListImpl unitList;
    /** the iterator of unitList */
    public ListIndex unitListIndex;
    /** the temporary address of an existing unit */
    public Unit aUnit;
    /** the activity space */
    public UnitActivitySpace unitActivitySpace;
    /** the min level of inter-visibility; below this level two units cannot
     *  exchange products; this value is not used in assignment coming from 
     *  OrderGenerator */
    public double interVisibilityMinLevel;

    public AssigningTool (Zone aZone, ListImpl ul, UnitActivitySpace uas, double iv)
    {
	super(aZone);

	// the list of the normal units
	unitList=ul;
	// its iterator
	unitListIndex=unitList.listBegin(getZone());
	// the unit activity space
	unitActivitySpace=uas;
	// the intervisibility min level
	interVisibilityMinLevel=iv;
    }


    public Order assign(Unit sendingUnit, Order firstOrder)
    {
	int i, ii, minWaitingList, minWaitingListPos;
	boolean unitNotFound, applied;

	// normal case
	unitNotFound=true;
	unitListIndex.setOffset(0);

	for (i=0;i<unitList.getCount() && unitNotFound ;i++)
	    {
		aUnit=(Unit)unitListIndex.get();
		if (firstOrder.getNextStep() ==	aUnit.getProductionPhase())
		    unitNotFound=false;

		unitListIndex.next();
	    }

	if(! unitNotFound)
	    {
		// same unit or units seeing each other above the min visibility level
		if(sendingUnit == aUnit || 
		   unitActivitySpace.interVisibility(sendingUnit, aUnit)>=
		   interVisibilityMinLevel)
		    aUnit.setWaitingList(firstOrder);
		return (Order) null;
	    }
	

	return firstOrder;

    }
    
    public Order assign(Order firstOrder)
    {
	int i, ii, minWaitingList, minWaitingListPos;
	boolean unitNotFound, applied;

	// normal case
	unitNotFound=true;
	unitListIndex.setOffset(0);

	for (i=0;i<unitList.getCount() && unitNotFound ;i++)
	    {
		aUnit=(Unit)unitListIndex.get();
		if (firstOrder.getNextStep() ==	aUnit.getProductionPhase())
		    unitNotFound=false;

		unitListIndex.next();
	    }

	if(! unitNotFound)
	    {
		aUnit.setWaitingList(firstOrder);
		return (Order) null;
	    }
	

	return firstOrder;

    }
}
